/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package a0b36pr2_du2.rychly_balik.model;

import a0b36pr2_du2.rychly_balik.model.doprava.Doprava;
import a0b36pr2_du2.rychly_balik.model.doprava.Riditelny;
import a0b36pr2_du2.rychly_balik.model.doprava.Riditelny.Opravneni;
import a0b36pr2_du2.rychly_balik.model.personalistika.AdministrativniPracovnik;
import a0b36pr2_du2.rychly_balik.model.personalistika.Pracovnik;
import a0b36pr2_du2.rychly_balik.model.personalistika.VykonnyPracovnik;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 * @author hapallen
 */
public class RychlyBalikDataService {

    Smena smena;
    VozovyPark vozovyPark;
    List<Zasilka> seznamZasilek = new ArrayList<Zasilka>();

    /**
     * nahraje do datoveho modelu startovaci data
     *
     */
    public void loadData() {
        vozovyPark = VozovyPark.createVozovyPark();
        Pracovnik p1 = new AdministrativniPracovnik("Olive", "Oyle");
        Pracovnik p2 = new VykonnyPracovnik("Pepek", "Namornik", Opravneni.KOLO);
        Pracovnik p3 = new VykonnyPracovnik("Bluto", "Brutus", Opravneni.C);
        smena = new Smena();
        smena.vloz(p2);
        smena.vloz(p1);
        smena.vloz(p3);
        try {
            seznamZasilek.add(new Zasilka(20, 50));
            Zasilka z = new Zasilka(30, 50);
            z.setPracovnik(p3);
            seznamZasilek.add(z);
        } catch (RychlyBalikException re) {
            System.out.println("Chyba pri plneni vseznamu zasilek.");
        }
        Smena s = new Smena();
        s.vloz(p2);
        s.vloz(p1);

    }

    public List<Zasilka> getSeznamZasilek() {
        return seznamZasilek;
    }

    public void setSeznamZasilek(List<Zasilka> seznamZasilek) {
        this.seznamZasilek = seznamZasilek;
    }

    public Smena getSmena() {
        return smena;
    }

    public void setSmena(Smena smena) {
        this.smena = smena;
    }

    public VozovyPark getVozovyPark() {
        return vozovyPark;
    }

    public void setVozovyPark(VozovyPark vozovyPark) {
        this.vozovyPark = vozovyPark;
    }

    /**
     *  urči seznam vozidel z vozového parku, která může směna obsluhovat
     * @param vozovyPark
     * @return
     */
    public List<Doprava> getVyuzitelneVozy() {
        List<Doprava> vyuzitelneVozy = new ArrayList<Doprava>();
        for (Doprava d : vozovyPark.getSeznam()) {
            //    try { // blok try-catch mohu pouzit misto  if( d instanceof Riditelny ){
            if (d instanceof Riditelny) {
                Riditelny r = (Riditelny) d; // if ()
                if (smena.getOpravneniSmeny().contains(r.getOpravneni())) {
                    vyuzitelneVozy.add(d);
                }
            }
            // } catch (java.lang.ClassCastException cce) {
            //monu nechat byt, protoze potrebuji jen riditelne
            // }
        }
        return vyuzitelneVozy;
    }

    /**
     * prida na seznam zasilek zasilku
     * @param zasilka
     */
    public void pridejZasilku(Zasilka zasilka) {
        seznamZasilek.add(zasilka);
    }

    /**
     * odebere zasilku
     * @param zasilka
     * @return odebrana zasilka
     * @throws RychlyBalikException pokud nelze zasilku odebrat
     */
    public Zasilka odeberZasilku(Zasilka zasilka) throws RychlyBalikException {
        if (seznamZasilek.remove(zasilka)) {
            return zasilka;
        }
        throw new RychlyBalikException("Tato zasilka v seznamu zásilek neexistuje.");
    }

    /**
     *
     * @return mnozina aktualne vyuzivanych pracovniku pro prepravu zasilek
     */
    public Set<VykonnyPracovnik> getVyuzivaniPracovnici() {
        Set<VykonnyPracovnik> pracovnici = new HashSet<VykonnyPracovnik>();
        for (Zasilka zasilka : seznamZasilek) {
            if (zasilka.getVykonnyPracovnik() != null) {
                pracovnici.add(zasilka.getVykonnyPracovnik());
            }
        }
        return pracovnici;
    }

    /**
     *prida do smeny pracovnika
     * @param pracovnik
     */
    public void pridejPracovnika(Pracovnik pracovnik) {
        smena.vloz(pracovnik);
    }

    /**   
     * odebere pracovnika ze smeny. Je mozne odebrat jen takoveho pracovnika,
     * ktery aktualne nema prirazenu zadnou zasilku.
     * @param pracovnik
     * @return odebrany pracovnik nebo null, pokud neexistuje
     * @throws RychlyBalikException pokud nelze zadaného pracovníka odebrat
     */
    public Pracovnik odeberPracovnika(Pracovnik pracovnik) throws RychlyBalikException {
        if (pracovnik == null) {
            throw new RychlyBalikException("Nebyl zadán žádný pracovník k odebrání");
        }
        if (pracovnik instanceof VykonnyPracovnik) {
            if (getVyuzivaniPracovnici().contains((VykonnyPracovnik) pracovnik)) {
                throw new RychlyBalikException("Nelze odebrat pracovníka, který aktuálně doručuje zásilku");
            }
        }
        return smena.odeber(pracovnik);
    }

    /**
     * vlozi do vozoveho parku dopravni prostredek
     * @param dopravniProstredek
     */
    public void pridejDopravniProstredek(Doprava dopravniProstredek) {
        vozovyPark.vloz(dopravniProstredek);
    }

    /**
     * odebere dopravní prostředek z vozového parku. Odebrat lze pouze takový dopravní prostředek,
     * aby bylo možné všechny zásilky příslušnými pracovníky s jejich oprávněním doručit. Tedy pro všechna využívaná oprávnění
     * musí vždy zůstat alespoň jeden dopravní prostředek.
     * @param dopravniProstredek
     * @return odebraný dopravní prostředek
     * @throws RychlyBalikException pokud nelze dopravní prostředek odebrat
     */
    public Doprava odeberDopravniProstredek(Doprava dopravniProstredek) throws RychlyBalikException {
        for (Zasilka zasilka : seznamZasilek) {
            if (getMoznouDopravuZasilkyPracovnikem(zasilka).contains(dopravniProstredek)) {
                // pokud je odebirany dopravni prostredek mezi temi, kterymi je mozne zasilku dopravit,
                //je potreba aby po jeho odebrani zustal jeste nejaky
                if (getMoznouDopravuZasilkyPracovnikem(zasilka).size() < 2) {
                    throw new RychlyBalikException("Neni mozne tento dopravni prostredek odebrat, protoze by nebylo mozno dorucit nejakou existujici zasilku");
                }
            }
        }
        return vozovyPark.odeberDopravniProstredek(dopravniProstredek);
    }

    /**
     * @param zasilka
     * @return sezanm dopravnich prostredku s dostateznou nostnosti pro zasilku
     */
    public Set<Doprava> getMoznouDopravuZasilky(Zasilka zasilka) {
        Set<Doprava> dopravniProstredky = new HashSet<Doprava>();
        Doprava dop = null;
        for (Doprava d : vozovyPark.getSeznam()) {
            if (zasilka.jeDPPouzitelny(d)) {
                dopravniProstredky.add(d);
            }
        }
        return dopravniProstredky;
    }

    /**
     *
     * @param zasilka
     * @return true, pokud je pracovnik schopen dorucit zadanou zasilku, jinak false
     */
    public boolean isPracovnikSchopenDorucit(Zasilka zasilka, Pracovnik pracovnik) throws RychlyBalikException {
        if (getMoznouDopravuZasilky(zasilka).isEmpty()) {
            throw new RychlyBalikException("Zasilku neni mozne dorucit, protoze pro ni neni vhodny prostredek ve vozovem parku.");
        }
        for (Doprava doprava : getMoznouDopravuZasilky(zasilka)) {
            if (pracovnik instanceof VykonnyPracovnik) {
                if (((VykonnyPracovnik) pracovnik).muzeRidit(doprava)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     *
     * @param zasilka
     * @return seznam dopravnich prostredku pouzitelnych pro dopravu teto zasilky jim prirazenym pracovnikem.
     * Pokud zasilka nema priraeneho pracovnika nelze ji dorucit a nema tedy ani dopravni prostredky
     */
    public Set<Doprava> getMoznouDopravuZasilkyPracovnikem(Zasilka zasilka) {
        Set<Doprava> dopravniProstredky = new HashSet<Doprava>();
        for (Doprava doprava : getMoznouDopravuZasilky(zasilka)) {
            if (zasilka.getVykonnyPracovnik() != null && zasilka.
                    getVykonnyPracovnik().muzeRidit(doprava)) {
                dopravniProstredky.add(doprava);
            }
        }
        return dopravniProstredky;
    }
}
